/*
	Impedance Chart - Impedance Chart using Canvas and Canvas Text.
	Copyright © 2007 Harry Whitfield

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the
	Free Software Foundation; either version 2 of the License, or (at your
	option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
	
	Impedance Chart - version 1.0.1
	8 November, 2007
	Copyright © 2007 Harry Whitfield
	mailto:g6auc@arrl.net
*/

// An Impedance Chart consists of two families of circles and circular arcs drawn in the (u,v) plane.

// The R-family circles have their centres at (u = r/(r+1), v = 0) with radius 1/(r+1),
// and with values of r running from 0.01 to 50. They all lie within the unit circle.

// The X-family circles have their centres at (u = 1, v = 1/x) with radius |1/x|,
// and with values of x running from -50 to -0.01, and from 0.01 to 50.
// They are clipped by the unit circle.

var window  = aux_window;	// was main_window
var hOffset =  40;
var vOffset =  50;
var width   = 590;	// was 594
var height  = 590;	// was 840
var zOrder  = 214;

var margins = 78; 	// was 72							// allowance for two margins
var scales  = 1.0;										// maximum radius of scales
var radius = ((width - margins) >> 1) / scales;

var dpi = 288;
var lw0 = (72 / dpi) / radius;
var lw1 = lw0;
var lw2 = 2 * lw0;
var lw3 = 3 * lw0;
var lw5 = 5 * lw0;

var	ImpChart = null;	// Canvas
var ctx      = null;	// Canvas 2d Context

///////////////////////////////// Start of the setStyle function /////////////////////////////////
Style.prototype.setStyle = function (styleString)
{
    var item, key, value;
    var items = styleString.split(';');
    for (var i = 0; i < items.length; i += 1)
    {
        item  = items[i].split(":");
        key   = item[0].replace(/^(\s*)(.*?)(\s*)$/, "$2");
        value = item[1].replace(/^(\s*)(.*?)(\s*)$/, "$2");
        
        switch(key)
        {
            case "background":              this.background           = value; break;
            case "background-attachment":   this.backgroundAttachment = value; break;
            case "background-color":        this.backgroundColor      = value; break;
            case "background-image":        this.backgroundImage      = value; break;
            case "background-position":     this.backgroundPosition   = value; break;
            case "background-repeat":       this.backgroundRepeat     = value; break;
            case "color":                   this.color                = value; break;
            case "font-family":             this.fontFamily           = value; break;
            case "font-size":               this.fontSize             = value; break;
            case "font-style":              this.fontStyle            = value; break;
            case "font-stretch":            this.fontStretch          = value; break;
            case "font-weight":             this.fontWeight           = value; break;
            case "opacity":                 this.opacity              = value; break;
            case "text-align":              this.textAlign            = value; break;
            case "text-decoration":         this.textDecoration       = value; break;
            case "-kon-background-fill":    this.KonBackgroundFill    = value; break;
            case "-kon-shadow":             this.KonShadow            = value; break;
            case "-kon-shadow-color":       this.KonShadowColor       = value; break;
            case "-kon-shadow-offset":      this.KonShadowOffset      = value; break;
            case "-kon-text-truncation":    this.KonTextTruncation    = value; break;
            default:                        break;
        }
    }
};
///////////////////////////////// End  of  the setStyle function /////////////////////////////////

///////////////////////////////// Start of the Canvas Text functions /////////////////////////////////
Canvas.prototype.getFullContext = function (s)
{
	var ctx = this.getContext( s );
	
	ctx.window = this.window;
	ctx.fontScaleX = 1;
	ctx.fontScaleY = 1;
	ctx.theStyle   = "font-family:Courier_New; font-size:12px; font-weight:normal; font-style:normal; opacity:1.0";	// default style
	ctx.textCount  = 0;
	
	ctx.scaleFont = function (scaleX, scaleY) { ctx.fontScaleX *= scaleX; ctx.fontScaleY *= scaleY; };
	ctx.setStyle  = function (style)          { ctx.theStyle = style; };

	ctx.show = function (x, y, string)
	{
    	var txt = new Text();
    	txt.window  = ctx.window;
    	txt.style.setStyle(ctx.theStyle);
    	txt.data    = string;
    	var width   = txt.width;
    	var height  = txt.height;
    	var path    = system.widgetDataFolder + "/show" + ctx.textCount;
    	txt.saveImageToFile( path, "png" );
    	txt.opacity = 0;
    	txt = null;

    	var img = new Image();
    	img.window = ctx.window;
    	img.opacity = 255;
    	img.src = path;
		ctx.drawImage( img, x, y, width/ctx.fontScaleX, height/ctx.fontScaleY );
		img.opacity = 0;
		filesystem.remove(path);
		img = null;
		ctx.textCount += 1;
		ctx.textCount %= 1024;
	};
	
	return ctx;
};
///////////////////////////////// End  of  the Canvas Text functions /////////////////////////////////

function newCanvas(window, hOffset, vOffset, width, height, zOrder)
{
	var o = new Canvas();
	o.hOffset = hOffset;
	o.vOffset = vOffset;
	o.width   = width;
	o.height  = height;
	o.zOrder  = zOrder;
	o.window  = window;
	return o;
}

function rdraw(r, f)									// draw R-family circles
{
	ctx.save();
	ctx.beginPath();
	if ((r %  100) === 0) { ctx.lineWidth = lw2; }
	if ((r % 1000) === 0) { ctx.lineWidth = lw3; }
	r = r * f;
	ctx.arc(r / (r + 1), 0, 1 / (r + 1), 0, 2*Math.PI, false);
	ctx.stroke();
	ctx.restore();
}

function rgrid1()
{
	var i, f = 0.001;
	ctx.lineWidth = lw1;
	for (i =  50; i <=  150; i +=  50) { rdraw(i, f); }
	for (i = 200; i <=  450; i +=  50) { rdraw(i, f); }
	for (i = 500; i <= 1000; i += 100) { rdraw(i, f); }
}

function rgrid2()
{
	var i, f = 0.01;
	ctx.lineWidth = lw1;
	for (i = 120; i <=  180; i +=  20) { rdraw(i, f); }
	for (i = 200; i <=  450; i +=  50) { rdraw(i, f); }
	for (i = 500; i <= 1000; i += 250) { rdraw(i, f); }
}

function rgrid3()
{
	var i, f = 0.1;
	ctx.lineWidth = lw1;
	for (i = 150; i <=  200; i +=  50) { rdraw(i, f); }
	for (i = 300; i <=  500; i += 100) { rdraw(i, f); }
}

function xdraw(r, f)									// draw X-family circles
{
	ctx.save();
	ctx.beginPath();
	if ((r %  100) === 0) { ctx.lineWidth = lw2; }
	if ((r % 1000) === 0) { ctx.lineWidth = lw3; }
	r = r * f;
	ctx.arc(1,  1/r, 1/r, Math.PI/2, 3*Math.PI/2, false);
	ctx.stroke();
	ctx.beginPath();
	ctx.arc(1, -1/r, 1/r,  Math.PI/2, 3*Math.PI/2, false);
	ctx.stroke();
	ctx.restore();
}

function xgrid1()
{
	var i, f = 0.001;
	ctx.lineWidth = lw1;
	for (i =  50; i <=  150; i +=  50) { xdraw(i, f); }
	for (i = 200; i <=  450; i +=  50) { xdraw(i, f); }
	for (i = 500; i <= 1000; i += 100) { xdraw(i, f); }
}

function xgrid2()
{
	var i, f = 0.01;
	ctx.lineWidth = lw1;
	for (i = 120; i <=  180; i +=  20) { xdraw(i, f); }
	for (i = 200; i <=  450; i +=  50) { xdraw(i, f); }
	for (i = 500; i <= 1000; i += 250) { xdraw(i, f); }
}

function xgrid3()
{
	var i, f = 0.1;
	ctx.lineWidth = lw1;
	for (i = 150; i <=  200; i +=  50) { xdraw(i, f); }
	for (i = 300; i <=  500; i += 100) { xdraw(i, f); }
}

function marker()
{
	ctx.beginPath();
	ctx.rect(20, 20, 20, 20);
	ctx.fillStyle = "#FF0000";	// red
	ctx.fill();
}

function makeBackground(width, height)
{
	ctx.beginPath();
	ctx.rect(0, 0, width, height);
	ctx.fillStyle = "#9F9F9F";	// grey
	ctx.fill();
	ctx.beginPath();
	ctx.rect(9, 9, width-18, height-18);
	ctx.fillStyle = "#FFFFFF";	// white
	ctx.fill();
}

function drawUnitCircle()
{
	ctx.beginPath();
	ctx.lineWidth = lw3;
	ctx.arc(0, 0, 1.0, 0, 2*Math.PI, false);
	ctx.stroke();
}

function drawAxis()										// draw horizontal diameter
{
	ctx.beginPath();
	ctx.lineWidth = lw3;
	ctx.moveTo(-1, 0);
	ctx.lineTo( 1, 0);
	ctx.stroke();
}

function drawCenterCircle()
{
	ctx.beginPath();
	ctx.lineWidth = lw3;
	ctx.arc(0, 0, 0.015, 0, 2*Math.PI, false);			// centre circle
	ctx.strokeStyle = "#000000";						// black
	ctx.stroke();

	ctx.beginPath();
	ctx.arc(0, 0, 0.0075, 0, 2*Math.PI, false);			// centre point
	ctx.fillStyle = "#000000";							// black
	ctx.fill();
}

function drawChart()
{
	ctx.strokeStyle = "#000000";						// black
	rgrid1();
	rgrid2();
	rgrid3();
	
	drawCenterCircle();
	
	ctx.strokeStyle = "#00000";							// black was blue
	xgrid1();
	xgrid2();
	xgrid3();
}

function swrite1(v)
{
	ctx.save();
	v = 0.1*v;
	var ang = Math.PI - 2*Math.atan2(v, 1);
	ctx.rotate(-ang);
	ctx.beginPath();
	ctx.show(0.935, -0.05, v.toFixed(1));
	ctx.restore();
}

function swrite2(v)
{
	ctx.save();
	var ang = Math.PI - 2*Math.atan2(v, 1);
	ctx.rotate(-ang);
	ctx.beginPath();
	ctx.show(0.95, -0.05, v.toFixed(0));
	ctx.restore();
}

function swrite3(v)
{
	ctx.save();
	v = 0.1*v;
	var ang = Math.PI + 2*Math.atan2(v, 1);
	ctx.rotate(-ang);
	ctx.beginPath();
	ctx.save();
	ctx.translate(0.99, 0.05);
	ctx.rotate(Math.PI);
	ctx.show(0, 0, v.toFixed(1));
	ctx.restore();
	ctx.restore();
}

function swrite4(v)
{
	ctx.save();
	var ang = Math.PI + 2*Math.atan2(v, 1);
	ctx.rotate(-ang);
	ctx.beginPath();
	ctx.save();
	ctx.translate(0.99, 0.05);
	ctx.rotate(Math.PI);
	ctx.show(0, 0, v.toFixed(0));
	ctx.restore();
	ctx.restore();
}

function swrite5(v)
{
	ctx.save();
	v = 0.1*v;
	var p = (v-1)/(v+1);
	ctx.rotate(-Math.PI/2);
	ctx.beginPath();
	ctx.save();
	ctx.translate(0, p);
	ctx.show(0.005, -0.045, v.toFixed(1));
	ctx.restore();
	ctx.restore();
}

function swrite6(v)
{
	ctx.save();
	var p = (v-1)/(v+1);
	ctx.rotate(-Math.PI/2);
	ctx.beginPath();
	ctx.save();
	ctx.translate(0, p);
	ctx.show(0.005, -0.045, v.toFixed(0));
	ctx.restore();
	ctx.restore();
}

function swrite7(v)
{
	ctx.save();
	ctx.translate(1,1);
	v = 0.1*v;
	var ang = 3*Math.PI/2 - 2*Math.atan2(1, v+1);
	ctx.rotate(ang);
	ctx.beginPath();
	ctx.save();
	ctx.translate(1, 0);
	ctx.show(-0.065, -0.045, v.toFixed(1));
	ctx.restore();
	ctx.restore();
}

function swrite8(v)
{
	ctx.save();
	ctx.translate(1, -1);
	v = 0.1*v;
	var ang = Math.PI/2 + 2*Math.atan2(1, v+1);
	ctx.rotate(ang);
	ctx.beginPath();
	ctx.save();
	ctx.translate(1, 0);
	ctx.rotate(Math.PI);
	ctx.show(0.005, -0.045, v.toFixed(1));
	ctx.restore();
	ctx.restore();
}

function swrite9(v)
{
	ctx.save();
	ctx.translate(0.5, 0);
	v = 0.1*v;
	var ang = Math.PI - 2*Math.atan2(v/2, 1);
	ctx.rotate(ang);
	ctx.beginPath();
	ctx.save();
	ctx.translate(0.495, 0.045);
	ctx.rotate(Math.PI);
	ctx.show(0, 0, v.toFixed(1));
	ctx.restore();
	ctx.restore();
}

function swrite10(v)
{
	ctx.save();
	ctx.translate(0.5, 0);
	v = 0.1*v;
	var ang = Math.PI + 2*Math.atan2(v/2, 1);
	ctx.rotate(ang);
	ctx.beginPath();
	ctx.save();
	ctx.translate(0.435, -0.045);
	ctx.show(0, 0, v.toFixed(1));
	ctx.restore();
	ctx.restore();
}

function scale3()
{
	var i;
	for (i =  1; i <=  9; i +=  1) { swrite1(i); }
	for (i = 10; i <= 18; i +=  2) { swrite1(i); }
	for (i = 20; i <= 50; i += 10) { swrite1(i); }
	for (i = 10; i <= 20; i += 10) { swrite2(i); }
	for (i = 50; i <= 50; i += 50) { swrite2(i); }
}

function scale4()
{
	var i;
	for (i =  1; i <=  9; i +=  1) { swrite3(i); }
	for (i = 10; i <= 18; i +=  2) { swrite3(i); }
	for (i = 20; i <= 50; i += 10) { swrite3(i); }
	for (i = 10; i <= 20; i += 10) { swrite4(i); }
	for (i = 50; i <= 50; i += 50) { swrite4(i); }
}

function scale5()
{
	var i;
	for (i =  1; i <=  9; i +=  1) { swrite5(i); }
	for (i = 10; i <= 18; i +=  2) { swrite5(i); }
	for (i = 20; i <= 50; i += 10) { swrite5(i); }
	for (i = 10; i <= 20; i += 10) { swrite6(i); }
	for (i = 50; i <= 50; i += 50) { swrite6(i); }
}

function scale7()
{
	for (var i =  2; i <= 10; i +=  2) { swrite7(i); }
}

function scale8()
{
	for (var i =  2; i <= 10; i +=  2) { swrite8(i); }
}

function scale9()
{
	for (var i =  2; i <= 10; i +=  2) { swrite9(i); }
}

function scale10()
{
	for (var i =  2; i <= 10; i +=  2) { swrite10(i); }
}

function getDrawColor()
{
	switch(preferences.drawColorPref.value)
	{
		case "Black": return "#000000";
		case "Blue":  return "#0000FF";
		case "Red":   return "#FF0000";
		default:      return "#000000";
	}
}

function plotCircle(rho)
{
	ctx.beginPath();
	ctx.lineWidth = lw5;
	ctx.arc(0, 0, rho, 0, 2*Math.PI, false);
	ctx.strokeStyle = getDrawColor();
	ctx.stroke();
}

function plotPoint(x, y)
{
	ctx.beginPath();
	ctx.lineWidth = lw5;
	ctx.arc(x, -y , 0.015, 0,2*Math.PI, false);
	ctx.strokeStyle = getDrawColor();
	ctx.stroke();
}

function removeChart()
{
	if (ImpChart) { window.removeChild(ImpChart); ImpChart = null; }
}

function openChart()
{
	removeChart();
	ImpChart   = newCanvas(window, hOffset, vOffset, width, height, zOrder);
	ctx = ImpChart.getFullContext( "2d");	
	makeBackground(width, height);
	//marker();

	ctx.save();
		ctx.translate(width>>1, height>>1);				// move origin to middle of page
		ctx.scale(radius, radius);						// scale unit circle to fit page
		drawUnitCircle();								// make it the clipping region
		ctx.save();
			ctx.clip();
			drawAxis();
			drawChart();
		ctx.restore();
		ctx.setStyle("font-family:Lucida_Grande; font-size:10px; font-weight:normal; font-style:normal; opacity:1.0");
		ctx.scaleFont(radius, radius);
		scale3();
		scale4();
		scale5();
		scale7();
		scale8();
		scale9();
		scale10();
		ctx.scaleFont(1/radius, 1/radius);
  //ctx.restore();
}

function plotChart()
{
	plotCircle(0.3333333);		// test plots to check orientation
	plotPoint(0.3333333,0.0);
	plotCircle(0.620174);
	plotPoint(0.538461824,0.307692467);
}

function closeChart()
{
	ctx.restore();
}

openChart();
//plotChart();
//closeChart();
//ImpChart.saveImageToFile(system.userDesktopFolder + "/" + widget.name + ".png", "png");
