// Netcalc - version 1.5
// 11 December, 2007
// Copyright 2000-2007 Harry Whitfield
// mailto:g6auc@arrl.net

include("Canvas.js");

var plotData = [];			// for EPS data
var pIndex = 0;

function drawCircle(rho)
{
  	var radius;
  	if (isFinite(rho.x) && isFinite(rho.y))
  	{
  		radius = rho.modulus();
  	
  		plotData[pIndex] = {};
  		plotData[pIndex].x = 0;
  		plotData[pIndex].y = 0;
  		plotData[pIndex].r = radius;
  		pIndex += 1;
  	
  		plotCircle(radius);
  	}
  	else { beep(); }
}

function placePoint(rho)
{
  	if (isFinite(rho.x) && isFinite(rho.y))
  	{
  		plotData[pIndex] = {};
  		plotData[pIndex].x = rho.x;
  		plotData[pIndex].y = rho.y;
  		plotData[pIndex].r = 0.016;
  		pIndex += 1;
  	
  		plotPoint(rho.x, rho.y);
  	}
  	else { beep(); }
}

function theEPSdata()
{
  	var xx = "";
  	var yy = "";
  	var rr = "";
  	var ps = "lw5 setlinewidth\r\n";
  	switch(preferences.drawColorPref.value)
  	{
  		case "Blue": 
  			ps += "nclBLUE\r\n";
  			break;
  		case "Red":
  			ps += "nclRED\r\n";
  			break;
  		default:
  			break;
  	}
  	ps += "newpath\r\n";
    for (var i =0; i < pIndex; i += 1)
    {
		xx = (plotData[i].x).toFixed(precision);
		yy = (plotData[i].y).toFixed(precision);
		rr = (plotData[i].r).toFixed(precision);
		
    	ps += xx + " " + yy + " " + rr + " 0 360 arc stroke\r\n";
  	}
  	ps += "grestore\r\n";
  	ps += "end\r\n";
  	ps += "showpage\r\n";
  	ps += "%%Trailer\r\n";
  	ps += "%%EOF\r\n";
  	return ps;
}

function showChart()
{
	ImpChart.zOrder = 214;
	
	CSw.zOrder = 215;
	ExportEPS.zOrder = 215;
	HideMe.zOrder = 215;
	SaveChart.zOrder = 215;
	SaveScreen.zOrder = 215;
	Prev2.zOrder = 215;
	Next2.zOrder = 215;
	
	aux_window.visible = true;
	ImpChart.visible = true;
	ImpChartFrame.visible = true;
	
	CSw.opacity = 255;
	ExportEPS.opacity = 255;
	HideMe.opacity = 255;
	SaveChart.opacity = 255;
	if (systemPlatform == "macintosh") { SaveScreen.opacity = 255; } else { SaveScreen.opacity = 0; }
	Prev2.opacity = 255;
	Next2.opacity = 255;
}

function hideChart()
{
	ImpChart.visible = false;
	ImpChartFrame.visible = false;
	aux_window.visible = false;
	
	CSw.opacity = 0;
	ExportEPS.opacity = 0;
	HideMe.opacity = 0;
	SaveChart.opacity = 0;
	SaveScreen.opacity = 0;
	Prev2.opacity = 0;
	Next2.opacity = 0;
	
	ImpChart.zOrder = 0;
	
	CSw.zOrder = 0;
	ExportEPS.zOrder = 0;
	HideMe.zOrder = 0;
	SaveChart.zOrder = 0;
	SaveScreen.zOrder = 0;
	Prev2.zOrder = 0;
	Next2.zOrder = 0;
}

function clearChart()
{
	openChart();
	for (i = 0; i < pIndex; i += 1)
	{
		delete plotData[i];
	}
  	pIndex = 0;
}

function btnVSWR()
{
	drawCircle(rho(toComplex(theXreg.data), toComplex(Z0reg.data)));
	if (ImpChart.visible) { showChart(); }
}

function btndot()
{
	placePoint(rho(toComplex(theXreg.data), toComplex(Z0reg.data)));
	if (ImpChart.visible) { showChart(); }
}

function btnS()
{
	showChart();
}

function btnCS()
{
	clearChart();
}

function btnCSw()
{
	clearChart();
}

function btnExportEPS()
{
	saveEPS( filesystem.readFile("ImpChart.ps") + theEPSdata() );
}

function btnHideMe()
{
	hideChart();
}

function validPath(path)
{
	var idx = path.lastIndexOf("/");
	if (idx < 0 ) { return false; }
	var dir = path.substring(0,idx);	// directory in which the file is to be stored must (still) exist
	if (filesystem.isDirectory(dir)) { return true; }
	beep();
	var answer = alert("Please specify an output file, then try again.");
	showWidgetPreferences();
	return false;
}

function btnSaveScreen()
{
	var path, result;
	if (system.platform == "macintosh")
	{
		path = preferences.scrFilePref.value;
		if (validPath(path))
		{
			result = runCommand('screencapture ' + path);
			if (result !== "")
			{
				beep();
				answer = alert("Save Screen failed. Please specify an output file, then try again.");
				showWidgetPreferences();
			}
		}
	}
	else { beep(); }
}

function saveEPS(totalStr)
{
	var result, answer;
	var path = preferences.epsFilePref.value;
	if (validPath(path))
	{
		result = filesystem.writeFile(path, totalStr);
		if (! result)
		{
			beep();
			answer = alert("Export EPS File failed. Please specify an output file, then try again.");
			showWidgetPreferences();
		}
	}
}

function btnSaveChart()
{
	var path = preferences.pngFilePref.value;
	if (validPath(path))
	{
		ImpChart.saveImageToFile(path, "png");
	}
}

function dummy() { }

function setChartPosition(overlap)
{
	var delta = 580;

	if (overlap)
	{
		aux_window.hOffset = main_window.hOffset;
		aux_window.vOffset = main_window.vOffset;
	}
	else
	{
		aux_window.hOffset = main_window.hOffset + delta;
		aux_window.vOffset = main_window.vOffset;
	}
}

function checkChart()
{
	setChartPosition(preferences.overlapPref.value == "On top of calculator");
}
