	// ∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞
	//
	// NetCalc - An RF Impedance Calculator
	// 
	// Copyright © 2000-2007 by Harry Whitfield, G6AUC. All Rights Reserved.
	// 
	// Broadly based on NETCALC - an RF Impedance Calculator by Ian White, G3SEK.
	// 
	// The concepts and command names follow NETCALC, but the coding and design
	// of this program are entirely new.
	// 
	// This program can also display impedances on a diagram similar to the
	// well-known Smith Chart.
	// 
	// The L-section code is based on a PERL program
	// by Claude Frantz <claude@pc0312a.rz.unibw-muenchen.de>
	// 
	// ∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞

	// Netcalc - version 1.5
	// 11 December, 2007
	// Copyright 2000-2007 Harry Whitfield
	// mailto:g6auc@arrl.net

	// The following flags control printing in the debug console window
		
	var eFlag = true;	// show general debug information
	var sFlag = true;	// show applescripts
	var lFlag = true;	// log runCommand() and appleScript() calls

	function eprint(theString) { if (eFlag) { print(theString); } }
	function sprint(theString) { if (sFlag) { print(theString); } }
	function lprint(theString) { if (lFlag) { log(theString);   } }

	var precision = 6;						// for use in toFixed(precision)
	
	include("Complex.js"); 					// #include Complex.js
	
	var FMultiplier = 0;					// to be set by menuAction
	var RMultiplier = 0;					// to be set by menuAction
	var LMultiplier = -6;					// to be set by menuAction
	var XMultiplier = 0;					// to be set by menuAction
	var CMultiplier = -12;					// to be set by menuAction
	
	var Mreg = [];							// Memory Registers 1..9
	
	var degreeSign = "°";
	var ohmSign = "Ω";
	
	include("tooltips.js"); 				// #include tooltips.js
	include("NetCalc.js"); 					// #include NetCalc.js
	include("Chart.js"); 					// #include Chart.js
	include("Menu.js"); 					// #include Menu.js

	function updatePrefs()
	{
		var delta = (3*(16 - preferences.fontSizePref.value))>>2;

		freq.vOffset    = 171 + delta;
		velfac.vOffset  = 171 + delta;
		resist.vOffset  =  57 + delta;
		induct.vOffset  =  86 + delta;
		react.vOffset   = 115 + delta;
		capacit.vOffset = 143 + delta;
		Factor.vOffset  = 171 + delta;
		Tlength.vOffset = 199 + delta;

		theYreg.font =
		theXreg.font =
		Preg.font    =
		Z0reg.font   =
		Nreg.font    =
		Rreg.font    =
		theZreg.font =
		freq.font    =
		velfac.font  =
		resist.font  =
		induct.font  =
		react.font   =
		capacit.font =
		Factor.font  =
		Tlength.font = preferences.generalFontPref.value;
	
		theYreg.size =
		theXreg.size =
		Preg.size    =
		Z0reg.size   =
		Nreg.size    =
		Rreg.size    =
		theZreg.size =
		freq.size    =
		velfac.size  =
		resist.size  =
		induct.size  =
		react.size   =
		capacit.size =
		Factor.size  =
		Tlength.size = preferences.fontSizePref.value;
	}
	
	function setTextBgOpacities(bgOpacity)
	{
		theYreg.bgOpacity =
		theXreg.bgOpacity =
		Preg.bgOpacity    =
		Z0reg.bgOpacity   =
		Nreg.bgOpacity    =
		Rreg.bgOpacity    =
		theZreg.bgOpacity =
		freq.bgOpacity    =
		velfac.bgOpacity  =
		resist.bgOpacity  =
		induct.bgOpacity  =
		react .bgOpacity  =
		capacit.bgOpacity =
		Factor.bgOpacity  =
		Tlength.bgOpacity = bgOpacity;
/*
		FMenu.setBgOpacity(0);
		RMenu.setBgOpacity(0);
		LMenu.setBgOpacity(0);
		XMenu.setBgOpacity(0);
		CMenu.setBgOpacity(0);
		RnMenu.setBgOpacity(0);
		QnMenu.setBgOpacity(0);
		MnMenu.setBgOpacity(0);
*/
	}

	// The following function is called when a menu item has been selected.

	function menuAction()	// (menuName, i, data)
	{
		var menuName = this.menuName, i = Number(this.i), data = this.data;
		
		eprint("Menu:" + menuName + " Index:" + i + " Data:" + data);
		
		var mlist = "";
		var item  = [];
		
		switch(menuName)
		{
		case 'FMenu':
			//list  = "Hz,kHz,MHz,GHz,THz";
			mlist = "0,3,6,9,12";
			item  = mlist.split(",");
			FMultiplier =  Number(item[i]);
			eprint("FMultiplier=" + FMultiplier);
			FMenu.text.data = data;
			break;
		case 'RMenu':
			//list  = "Ω,kΩ,MΩ,GΩ";
			mlist = "0,3,6,9";
			item  = mlist.split(",");
			RMultiplier =  Number(item[i]);
			eprint("RMultiplier=" + RMultiplier);
			RMenu.text.data = data;
			break;
		case 'LMenu':
			//list  = "pH,nH,µH,mH,H,kH";
			mlist = "-12,-9,-6,-3,0,3,6,9";
			item  = mlist.split(",");
			LMultiplier =  Number(item[i]);
			eprint("LMultiplier=" + LMultiplier);
			LMenu.text.data = data;
			break;
		case 'XMenu':
			//list  = "Ω,kΩ,MΩ,GΩ";
			mlist = "0,3,6,9";
			item  = mlist.split(",");
			XMultiplier =  Number(item[i]);
			eprint("XMultiplier=" + XMultiplier);
			XMenu.text.data = data;
			break;
		case 'CMenu':
			//list  = "pF,nF,µF,mF,F,kF";
			mlist = "-12,-9,-6,-3,0,3,6,9";
			item  = mlist.split(",");
			CMultiplier =  Number(item[i]);
			eprint("CMultiplier=" + CMultiplier);
			CMenu.text.data = data;
			break;
		case 'RnMenu':
			menuRn(i);
			break;
		case 'QnMenu':
			menuQn(i);
			break;
		case 'MnMenu':
			menuMn(i);
			break;
		default:
			beep();
			break;
		}
	}

	var Farray = ['Hz','kHz','MHz','GHz','THz'];
	var Rarray = ['Ω','kΩ','MΩ','GΩ'];
	var Larray = ['pH','nH','µH','mH','H','kH'];
	var Carray = ['pF','nF','µF','mF','F','kF'];
	
	if (system.platform == "windows") { Rarray = ['ohm','k','M','G']; }
	
	var FMenu  = new Menu(main_window, 'FMenu',  Farray, 0, 164, 171, 56, 20, "#8bc48b", "bgGreen.png", 104, "Set frequency multiplier here.",   true);
	var RMenu  = new Menu(main_window, 'RMenu',  Rarray, 0, 520,  57, 56, 20, "#dddddd", "bgGrey.png",  110, "Set resistance multiplier here.",  true);
	var LMenu  = new Menu(main_window, 'LMenu',  Larray, 2, 520,  86, 56, 20, "#dddddd", "bgGrey.png",  108, "Set inductance multiplier here.",  true);
	var XMenu  = new Menu(main_window, 'XMenu',  Rarray, 0, 520, 115, 56, 20, "#dddddd", "bgGrey.png",  106, "Set reactance multiplier here.",   true);
	var CMenu  = new Menu(main_window, 'CMenu',  Carray, 0, 520, 143, 56, 20, "#dddddd", "bgGrey.png",  104, "Set capacitance multiplier here.", true);
	var RnMenu = new Menu(main_window, 'RnMenu', ['0','1','2','3','4','5','6','7','8','9'], -1, 378, 241, 35, 22, "#ffff00", "Rn.png",      108, "Pushes the  value of memory n (1..9) onto the stack (at X).  Does not change memory n.", false);
	var QnMenu = new Menu(main_window, 'QnMenu', ['0','1','2','3','4','5','6','7','8','9'], -1, 378, 272, 35, 22, "#66ccff", "Qn.png",      106, "Displays the value of memory n (1..9) in Z. Does not change the stack or memory n.",     false);
	var MnMenu = new Menu(main_window, 'MnMenu', ['0','1','2','3','4','5','6','7','8','9'], -1, 378, 304, 35, 22, "#00ffff", "Mn.png",      104, "Copies the X register to memory register n (1..9). Does not change X.",                  false);
		
	for (var i = 0; i < 10; i += 1) { Mreg[i] = "0,0";}

	hideChart();
	setChartPosition(preferences.overlapPref.value == "On top of calculator");
	update();
	
	if (preferences.scrFilePref.value === "") { preferences.scrFilePref.value = system.userDesktopFolder + "/NetCalc.png";  }
	if (preferences.epsFilePref.value === "") { preferences.epsFilePref.value = system.userDesktopFolder + "/ImpChart.eps"; }
	if (preferences.pngFilePref.value === "") { preferences.pngFilePref.value = system.userDesktopFolder + "/ImpChart.png"; }

	updatePrefs();
	setTextBgOpacities(preferences.textBgOpacityPref.value);

	//main_window.visible = true;
